@@ -3,12 +3,12 @@
  * btinsert.c
  *	  Item insertion in Lehman and Yao btrees for Postgres.
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/access/nbtree/nbtinsert.c,v 1.88 2002/01/01 20:32:37 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/access/nbtree/nbtinsert.c,v 1.96.2.1 2003/02/21 18:24:54 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -119,14 +119,14 @@ top:
 	 *
 	 * NOTE: obviously, _bt_check_unique can only detect keys that are
 	 * already in the index; so it cannot defend against concurrent
-	 * insertions of the same key.  We protect against that by means
-	 * of holding a write lock on the target page.  Any other would-be
+	 * insertions of the same key.	We protect against that by means of
+	 * holding a write lock on the target page.  Any other would-be
 	 * inserter of the same key must acquire a write lock on the same
 	 * target page, so only one would-be inserter can be making the check
-	 * at one time.  Furthermore, once we are past the check we hold
-	 * write locks continuously until we have performed our insertion,
-	 * so no later inserter can fail to see our insertion.  (This
-	 * requires some care in _bt_insertonpg.)
+	 * at one time.  Furthermore, once we are past the check we hold write
+	 * locks continuously until we have performed our insertion, so no
+	 * later inserter can fail to see our insertion.  (This requires some
+	 * care in _bt_insertonpg.)
 	 *
 	 * If we must wait for another xact, we release the lock while waiting,
 	 * and then must start over completely.
@@ -176,7 +176,6 @@ _bt_check_unique(Relation rel, BTItem btitem, Relation heapRel,
 	Page		page;
 	BTPageOpaque opaque;
 	Buffer		nbuf = InvalidBuffer;
-	bool		chtup = true;
 
 	page = BufferGetPage(buf);
 	opaque = (BTPageOpaque) PageGetSpecialPointer(page);
@@ -194,70 +193,87 @@ _bt_check_unique(Relation rel, BTItem btitem, Relation heapRel,
 	for (;;)
 	{
 		HeapTupleData htup;
-		Buffer		buffer;
+		Buffer		hbuffer;
+		ItemId		curitemid;
 		BTItem		cbti;
 		BlockNumber nblkno;
 
 		/*
-		 * _bt_compare returns 0 for (1,NULL) and (1,NULL) - this's how we
-		 * handling NULLs - and so we must not use _bt_compare in real
-		 * comparison, but only for ordering/finding items on pages. -
-		 * vadim 03/24/97
-		 *
 		 * make sure the offset points to an actual key before trying to
 		 * compare it...
 		 */
 		if (offset <= maxoff)
 		{
+			/*
+			 * _bt_compare returns 0 for (1,NULL) and (1,NULL) - this's
+			 * how we handling NULLs - and so we must not use _bt_compare
+			 * in real comparison, but only for ordering/finding items on
+			 * pages. - vadim 03/24/97
+			 */
 			if (!_bt_isequal(itupdesc, page, offset, natts, itup_scankey))
 				break;			/* we're past all the equal tuples */
 
+			curitemid = PageGetItemId(page, offset);
+
 			/*
-			 * Have to check is inserted heap tuple deleted one (i.e. just
-			 * moved to another place by vacuum)!  We only need to do this
-			 * once, but don't want to do it at all unless we see equal
-			 * tuples, so as not to slow down unequal case.
+			 * We can skip the heap fetch if the item is marked killed.
 			 */
-			if (chtup)
+			if (!ItemIdDeleted(curitemid))
 			{
-				htup.t_self = btitem->bti_itup.t_tid;
-				heap_fetch(heapRel, SnapshotDirty, &htup, &buffer, NULL);
-				if (htup.t_data == NULL)		/* YES! */
-					break;
-				/* Live tuple is being inserted, so continue checking */
-				ReleaseBuffer(buffer);
-				chtup = false;
-			}
-
-			cbti = (BTItem) PageGetItem(page, PageGetItemId(page, offset));
-			htup.t_self = cbti->bti_itup.t_tid;
-			heap_fetch(heapRel, SnapshotDirty, &htup, &buffer, NULL);
-			if (htup.t_data != NULL)	/* it is a duplicate */
-			{
-				TransactionId xwait =
-				(TransactionIdIsValid(SnapshotDirty->xmin)) ?
-				SnapshotDirty->xmin : SnapshotDirty->xmax;
-
-				/*
-				 * If this tuple is being updated by other transaction
-				 * then we have to wait for its commit/abort.
-				 */
-				ReleaseBuffer(buffer);
-				if (TransactionIdIsValid(xwait))
+				cbti = (BTItem) PageGetItem(page, curitemid);
+				htup.t_self = cbti->bti_itup.t_tid;
+				if (heap_fetch(heapRel, SnapshotDirty, &htup, &hbuffer,
+							   true, NULL))
 				{
-					if (nbuf != InvalidBuffer)
-						_bt_relbuf(rel, nbuf);
-					/* Tell _bt_doinsert to wait... */
-					return xwait;
-				}
+					/* it is a duplicate */
+					TransactionId xwait =
+					(TransactionIdIsValid(SnapshotDirty->xmin)) ?
+					SnapshotDirty->xmin : SnapshotDirty->xmax;
 
-				/*
-				 * Otherwise we have a definite conflict.
-				 */
-				elog(ERROR, "Cannot insert a duplicate key into unique index %s",
-					 RelationGetRelationName(rel));
+					ReleaseBuffer(hbuffer);
+
+					/*
+					 * If this tuple is being updated by other transaction
+					 * then we have to wait for its commit/abort.
+					 */
+					if (TransactionIdIsValid(xwait))
+					{
+						if (nbuf != InvalidBuffer)
+							_bt_relbuf(rel, nbuf);
+						/* Tell _bt_doinsert to wait... */
+						return xwait;
+					}
+
+					/*
+					 * Otherwise we have a definite conflict.
+					 */
+					elog(ERROR, "Cannot insert a duplicate key into unique index %s",
+						 RelationGetRelationName(rel));
+				}
+				else if (htup.t_data != NULL)
+				{
+					/*
+					 * Hmm, if we can't see the tuple, maybe it can be
+					 * marked killed.  This logic should match
+					 * index_getnext and btgettuple.
+					 */
+					uint16		sv_infomask;
+
+					LockBuffer(hbuffer, BUFFER_LOCK_SHARE);
+					sv_infomask = htup.t_data->t_infomask;
+					if (HeapTupleSatisfiesVacuum(htup.t_data,
+												 RecentGlobalXmin) ==
+						HEAPTUPLE_DEAD)
+					{
+						curitemid->lp_flags |= LP_DELETE;
+						SetBufferCommitInfoNeedsSave(buf);
+					}
+					if (sv_infomask != htup.t_data->t_infomask)
+						SetBufferCommitInfoNeedsSave(hbuffer);
+					LockBuffer(hbuffer, BUFFER_LOCK_UNLOCK);
+					ReleaseBuffer(hbuffer);
+				}
 			}
-			/* htup null so no buffer to release */
 		}
 
 		/*
@@ -369,10 +385,9 @@ _bt_insertonpg(Relation rel,
 	 * to 1/3 the per-page available space. Note that at this point,
 	 * itemsz doesn't include the ItemId.
 	 */
-	if (itemsz > (PageGetPageSize(page) - sizeof(PageHeaderData) - MAXALIGN(sizeof(BTPageOpaqueData))) / 3 - sizeof(ItemIdData))
+	if (itemsz > BTMaxItemSize(page))
 		elog(ERROR, "btree: index item size %lu exceeds maximum %lu",
-			 (unsigned long) itemsz,
-			 (PageGetPageSize(page) - sizeof(PageHeaderData) - MAXALIGN(sizeof(BTPageOpaqueData))) / 3 - sizeof(ItemIdData));
+			 (unsigned long) itemsz, BTMaxItemSize(page));
 
 	/*
 	 * Determine exactly where new item will go.
@@ -408,7 +423,7 @@ _bt_insertonpg(Relation rel,
 		{
 			/* step right one page */
 			BlockNumber rblkno = lpageop->btpo_next;
-			Buffer	rbuf;
+			Buffer		rbuf;
 
 			/*
 			 * must write-lock next page before releasing write lock on
@@ -507,7 +522,7 @@ _bt_insertonpg(Relation rel,
 			/* If root page was splitted */
 			if (stack == (BTStack) NULL)
 			{
-				elog(DEBUG, "btree: concurrent ROOT page split");
+				elog(LOG, "btree: concurrent ROOT page split");
 
 				/*
 				 * If root page splitter failed to create new root page
@@ -520,7 +535,7 @@ _bt_insertonpg(Relation rel,
 						elog(ERROR, "bt_insertonpg[%s]: no root page found", RelationGetRelationName(rel));
 					_bt_wrtbuf(rel, rbuf);
 					_bt_wrtnorelbuf(rel, buf);
-					elog(NOTICE, "bt_insertonpg[%s]: root page unfound - fixing upper levels", RelationGetRelationName(rel));
+					elog(WARNING, "bt_insertonpg[%s]: root page unfound - fixing upper levels", RelationGetRelationName(rel));
 					_bt_fixup(rel, buf);
 					goto formres;
 				}
@@ -570,7 +585,7 @@ _bt_insertonpg(Relation rel,
 					elog(ERROR, "_bt_getstackbuf: my bits moved right off the end of the world!"
 						 "\n\tRecreate index %s.", RelationGetRelationName(rel));
 				pfree(new_item);
-				elog(NOTICE, "bt_insertonpg[%s]: parent page unfound - fixing branch", RelationGetRelationName(rel));
+				elog(WARNING, "bt_insertonpg[%s]: parent page unfound - fixing branch", RelationGetRelationName(rel));
 				_bt_fixbranch(rel, bknum, rbknum, stack);
 				goto formres;
 			}
@@ -610,8 +625,11 @@ _bt_insertuple(Relation rel, Buffer buf,
 	BTPageOpaque pageop = (BTPageOpaque) PageGetSpecialPointer(page);
 
 	START_CRIT_SECTION();
+
 	_bt_pgaddtup(rel, page, itemsz, btitem, newitemoff, "page");
+
 	/* XLOG stuff */
+	if (!rel->rd_istemp)
 	{
 		xl_btree_insert xlrec;
 		uint8		flag = XLOG_BTREE_INSERT;
@@ -681,6 +699,7 @@ _bt_split(Relation rel, Buffer buf, OffsetNumber firstright,
 				oopaque;
 	Buffer		sbuf = 0;
 	Page		spage = 0;
+	BTPageOpaque sopaque = 0;
 	Size		itemsz;
 	ItemId		itemid;
 	BTItem		item;
@@ -735,7 +754,7 @@ _bt_split(Relation rel, Buffer buf, OffsetNumber firstright,
 		item = (BTItem) PageGetItem(origpage, itemid);
 		if (PageAddItem(rightpage, (Item) item, itemsz, rightoff,
 						LP_USED) == InvalidOffsetNumber)
-			elog(STOP, "btree: failed to add hikey to the right sibling");
+			elog(PANIC, "btree: failed to add hikey to the right sibling");
 		rightoff = OffsetNumberNext(rightoff);
 	}
 
@@ -761,7 +780,7 @@ _bt_split(Relation rel, Buffer buf, OffsetNumber firstright,
 	lhikey = item;
 	if (PageAddItem(leftpage, (Item) item, itemsz, leftoff,
 					LP_USED) == InvalidOffsetNumber)
-		elog(STOP, "btree: failed to add hikey to the left sibling");
+		elog(PANIC, "btree: failed to add hikey to the left sibling");
 	leftoff = OffsetNumberNext(leftoff);
 
 	/*
@@ -844,6 +863,7 @@ _bt_split(Relation rel, Buffer buf, OffsetNumber firstright,
 	{
 		sbuf = _bt_getbuf(rel, ropaque->btpo_next, BT_WRITE);
 		spage = BufferGetPage(sbuf);
+		sopaque = (BTPageOpaque) PageGetSpecialPointer(spage);
 	}
 
 	/*
@@ -853,6 +873,12 @@ _bt_split(Relation rel, Buffer buf, OffsetNumber firstright,
 	 * NO ELOG(ERROR) till right sibling is updated.
 	 */
 	START_CRIT_SECTION();
+
+	if (!P_RIGHTMOST(ropaque))
+		sopaque->btpo_prev = BufferGetBlockNumber(rbuf);
+
+	/* XLOG stuff */
+	if (!rel->rd_istemp)
 	{
 		xl_btree_split xlrec;
 		int			flag = (newitemonleft) ?
@@ -878,7 +904,7 @@ _bt_split(Relation rel, Buffer buf, OffsetNumber firstright,
 		BlockIdSet(&(xlrec.rightblk), ropaque->btpo_next);
 
 		/*
-		 * Dirrect access to page is not good but faster - we should
+		 * Direct access to page is not good but faster - we should
 		 * implement some new func in page API.
 		 */
 		xlrec.leftlen = ((PageHeader) leftpage)->pd_special -
@@ -901,10 +927,6 @@ _bt_split(Relation rel, Buffer buf, OffsetNumber firstright,
 
 		if (!P_RIGHTMOST(ropaque))
 		{
-			BTPageOpaque sopaque = (BTPageOpaque) PageGetSpecialPointer(spage);
-
-			sopaque->btpo_prev = BufferGetBlockNumber(rbuf);
-
 			rdata[2].next = &(rdata[3]);
 			rdata[3].buffer = sbuf;
 			rdata[3].data = NULL;
@@ -1006,9 +1028,8 @@ _bt_findsplitloc(Relation rel,
 
 	/* Total free space available on a btree page, after fixed overhead */
 	leftspace = rightspace =
-		PageGetPageSize(page) - sizeof(PageHeaderData) -
-		MAXALIGN(sizeof(BTPageOpaqueData))
-		+sizeof(ItemIdData);
+		PageGetPageSize(page) - SizeOfPageHeaderData -
+		MAXALIGN(sizeof(BTPageOpaqueData));
 
 	/*
 	 * Finding the best possible split would require checking all the
@@ -1316,7 +1337,7 @@ _bt_newroot(Relation rel, Buffer lbuf, Buffer rbuf)
 	 * the two items will go into positions P_HIKEY and P_FIRSTKEY.
 	 */
 	if (PageAddItem(rootpage, (Item) new_item, itemsz, P_HIKEY, LP_USED) == InvalidOffsetNumber)
-		elog(STOP, "btree: failed to add leftkey to new root page");
+		elog(PANIC, "btree: failed to add leftkey to new root page");
 	pfree(new_item);
 
 	/*
@@ -1333,13 +1354,14 @@ _bt_newroot(Relation rel, Buffer lbuf, Buffer rbuf)
 	 * insert the right page pointer into the new root page.
 	 */
 	if (PageAddItem(rootpage, (Item) new_item, itemsz, P_FIRSTKEY, LP_USED) == InvalidOffsetNumber)
-		elog(STOP, "btree: failed to add rightkey to new root page");
+		elog(PANIC, "btree: failed to add rightkey to new root page");
 	pfree(new_item);
 
 	metad->btm_root = rootblknum;
 	(metad->btm_level)++;
 
 	/* XLOG stuff */
+	if (!rel->rd_istemp)
 	{
 		xl_btree_newroot xlrec;
 		XLogRecPtr	recptr;
@@ -1354,7 +1376,7 @@ _bt_newroot(Relation rel, Buffer lbuf, Buffer rbuf)
 		rdata[0].next = &(rdata[1]);
 
 		/*
-		 * Dirrect access to page is not good but faster - we should
+		 * Direct access to page is not good but faster - we should
 		 * implement some new func in page API.
 		 */
 		rdata[1].buffer = InvalidBuffer;
@@ -1376,6 +1398,7 @@ _bt_newroot(Relation rel, Buffer lbuf, Buffer rbuf)
 		PageSetLSN(rpage, recptr);
 		PageSetSUI(rpage, ThisStartUpID);
 	}
+
 	END_CRIT_SECTION();
 
 	/* write and let go of metapage buffer */
@@ -1574,7 +1597,7 @@ _bt_fixtree(Relation rel, BlockNumber blkno)
 			/* Call _bt_fixroot() if there is no upper level */
 			if (BTreeInvalidParent(opaque))
 			{
-				elog(NOTICE, "bt_fixtree[%s]: fixing root page", RelationGetRelationName(rel));
+				elog(WARNING, "bt_fixtree[%s]: fixing root page", RelationGetRelationName(rel));
 				buf = _bt_fixroot(rel, buf, true);
 				_bt_relbuf(rel, buf);
 				return;
@@ -1925,7 +1948,7 @@ _bt_fixbranch(Relation rel, BlockNumber lblkno,
 		break;
 	}
 
-	elog(NOTICE, "bt_fixbranch[%s]: fixing upper levels", RelationGetRelationName(rel));
+	elog(WARNING, "bt_fixbranch[%s]: fixing upper levels", RelationGetRelationName(rel));
 	_bt_fixup(rel, buf);
 
 	return;
@@ -1956,7 +1979,7 @@ _bt_fixup(Relation rel, Buffer buf)
 		{
 			blkno = opaque->btpo_parent;
 			_bt_relbuf(rel, buf);
-			elog(NOTICE, "bt_fixup[%s]: checking/fixing upper levels", RelationGetRelationName(rel));
+			elog(WARNING, "bt_fixup[%s]: checking/fixing upper levels", RelationGetRelationName(rel));
 			_bt_fixtree(rel, blkno);
 			return;
 		}
@@ -1971,7 +1994,7 @@ _bt_fixup(Relation rel, Buffer buf)
 	 * Ok, we are on the leftmost page, it's write locked by us and its
 	 * btpo_parent points to meta page - time for _bt_fixroot().
 	 */
-	elog(NOTICE, "bt_fixup[%s]: fixing root page", RelationGetRelationName(rel));
+	elog(WARNING, "bt_fixup[%s]: fixing root page", RelationGetRelationName(rel));
 	buf = _bt_fixroot(rel, buf, true);
 	_bt_relbuf(rel, buf);
 }
@@ -2034,7 +2057,7 @@ _bt_pgaddtup(Relation rel,
 
 	if (PageAddItem(page, (Item) btitem, itemsize, itup_off,
 					LP_USED) == InvalidOffsetNumber)
-		elog(STOP, "btree: failed to add item to the %s for %s",
+		elog(PANIC, "btree: failed to add item to the %s for %s",
 			 where, RelationGetRelationName(rel));
 }
 
