@@ -11,7 +11,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/libpq/be-secure.c,v 1.15.2.4 2002/12/18 13:20:00 pgsql Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/libpq/be-secure.c,v 1.15.2.11 2003/04/10 23:03:13 tgl Exp $
  *
  *	  Since the server static private key ($DataDir/server.key)
  *	  will normally be stored unencrypted so that the database
@@ -116,7 +116,7 @@ void		secure_destroy(void);
 int			secure_open_server(Port *);
 void		secure_close(Port *);
 ssize_t		secure_read(Port *, void *ptr, size_t len);
-ssize_t		secure_write(Port *, const void *ptr, size_t len);
+ssize_t		secure_write(Port *, void *ptr, size_t len);
 
 #ifdef USE_SSL
 static DH  *load_dh_file(int keylength);
@@ -273,12 +273,7 @@ secure_read(Port *port, void *ptr, size_t len)
 #ifdef USE_SSL
 	if (port->ssl)
 	{
-		if (port->count > RENEGOTIATION_LIMIT)
-		{
-			SSL_renegotiate(port->ssl);
-			port->count = 0;
-		}
-
+	rloop:
 		n = SSL_read(port->ssl, ptr, len);
 		switch (SSL_get_error(port->ssl, n))
 		{
@@ -286,10 +281,13 @@ secure_read(Port *port, void *ptr, size_t len)
 				port->count += n;
 				break;
 			case SSL_ERROR_WANT_READ:
-				break;
+			case SSL_ERROR_WANT_WRITE:
+				goto rloop;
 			case SSL_ERROR_SYSCALL:
 				if (n == -1)
-					elog(COMMERROR, "SSL SYSCALL error: %s", strerror(errno));
+					elog(COMMERROR, "SSL SYSCALL error: %m");
+				else
+					elog(COMMERROR, "SSL SYSCALL error: EOF detected");
 				break;
 			case SSL_ERROR_SSL:
 				elog(COMMERROR, "SSL error: %s", SSLerrmessage());
@@ -299,6 +297,9 @@ secure_read(Port *port, void *ptr, size_t len)
 				errno = ECONNRESET;
 				n = -1;
 				break;
+			default:
+				elog(COMMERROR, "Unknown SSL error code");
+				break;
 		}
 	}
 	else
@@ -312,7 +313,7 @@ secure_read(Port *port, void *ptr, size_t len)
  *	Write data to a secure connection.
  */
 ssize_t
-secure_write(Port *port, const void *ptr, size_t len)
+secure_write(Port *port, void *ptr, size_t len)
 {
 	ssize_t		n;
 
@@ -325,21 +326,33 @@ secure_write(Port *port, const void *ptr, size_t len)
 	{
 		if (port->count > RENEGOTIATION_LIMIT)
 		{
-			SSL_renegotiate(port->ssl);
+			SSL_set_session_id_context(port->ssl, (void *) &SSL_context,
+									   sizeof(SSL_context));
+			if (SSL_renegotiate(port->ssl) <= 0)
+				elog(COMMERROR, "SSL renegotiation failure");
+			if (SSL_do_handshake(port->ssl) <= 0)
+				elog(COMMERROR, "SSL renegotiation failure");
+			port->ssl->state = SSL_ST_ACCEPT;
+			if (SSL_do_handshake(port->ssl) <= 0)
+				elog(COMMERROR, "SSL renegotiation failure");
 			port->count = 0;
 		}
 
+	wloop:
 		n = SSL_write(port->ssl, ptr, len);
 		switch (SSL_get_error(port->ssl, n))
 		{
 			case SSL_ERROR_NONE:
 				port->count += n;
 				break;
+			case SSL_ERROR_WANT_READ:
 			case SSL_ERROR_WANT_WRITE:
-				break;
+				goto wloop;
 			case SSL_ERROR_SYSCALL:
 				if (n == -1)
-					elog(COMMERROR, "SSL SYSCALL error: %s", strerror(errno));
+					elog(COMMERROR, "SSL SYSCALL error: %m");
+				else
+					elog(COMMERROR, "SSL SYSCALL error: EOF detected");
 				break;
 			case SSL_ERROR_SSL:
 				elog(COMMERROR, "SSL error: %s", SSLerrmessage());
@@ -349,6 +362,9 @@ secure_write(Port *port, const void *ptr, size_t len)
 				errno = ECONNRESET;
 				n = -1;
 				break;
+			default:
+				elog(COMMERROR, "Unknown SSL error code");
+				break;
 		}
 	}
 	else
@@ -616,7 +632,9 @@ initialize_SSL(void)
 		if (!S_ISREG(buf.st_mode) || (buf.st_mode & 0077) ||
 			buf.st_uid != getuid())
 		{
-			postmaster_error("bad permissions on private key file (%s)", fnbuf);
+			postmaster_error("bad permissions on private key file (%s)\n"
+"File must be owned by the proper user and must have no permissions for\n"
+"\"group\" or \"other\".", fnbuf);
 			ExitPostmaster(1);
 		}
 		if (!SSL_CTX_use_PrivateKey_file(SSL_context, fnbuf, SSL_FILETYPE_PEM))
@@ -635,7 +653,7 @@ initialize_SSL(void)
 
 	/* set up empheral DH keys */
 	SSL_CTX_set_tmp_dh_callback(SSL_context, tmp_dh_cb);
-	SSL_CTX_set_options(SSL_context, SSL_OP_SINGLE_DH_USE);
+	SSL_CTX_set_options(SSL_context, SSL_OP_SINGLE_DH_USE | SSL_OP_NO_SSLv2);
 
 	/* accept client certificates, but don't require them. */
 	snprintf(fnbuf, sizeof fnbuf, "%s/root.crt", DataDir);
