@@ -3,16 +3,16 @@
  *
  * Copyright 2000 by PostgreSQL Global Development Group
  *
- * $Header: /cvsroot/pgsql-server/src/bin/psql/help.h,v 1.8 2001/10/28 06:25:58 momjian Exp $
+ * $Header: /cvsroot/pgsql-server/src/bin/psql/help.h,v 1.10 2002/10/23 19:23:57 momjian Exp $
  */
 #ifndef HELP_H
 #define HELP_H
 
 void		usage(void);
 
-void		slashUsage(void);
+void		slashUsage(bool pager);
 
-void		helpSQL(const char *topic);
+void		helpSQL(const char *topic, bool pager);
 
 void		print_copyright(void);
 
