@@ -1,9 +1,9 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright 2000 by PostgreSQL Global Development Group
+ * Copyright 2000-2002 by PostgreSQL Global Development Group
  *
- * $Header: /cvsroot/pgsql-server/src/bin/psql/tab-complete.c,v 1.41 2001/11/05 17:46:31 momjian Exp $
+ * $Header: /cvsroot/pgsql-server/src/bin/psql/tab-complete.c,v 1.64 2002/09/04 20:31:36 momjian Exp $
  */
 
 /*----------------------------------------------------------------------
@@ -34,8 +34,7 @@
  *	 buffer rather than readline's line buffer, which would require
  *	 some major revisions of things.)
  *
- * - Table or attribute names with spaces in it will equally confuse
- *	 it.
+ * - Table or attribute names with spaces in it may confuse it.
  *
  * - Quotes, parenthesis, and other funny characters are not handled
  *	 all that gracefully.
@@ -119,11 +118,20 @@ initialize_readline(void)
 }
 
 
+/*
+ * Queries to get lists of names of various kinds of things, possibly
+ * restricted to names matching a partially entered name.  In these queries,
+ * the %s will be replaced by the text entered so far, the %d by its length.
+ */
+
+#define Query_for_list_of_tables "SELECT relname FROM pg_catalog.pg_class WHERE (relkind='r' or relkind='v') and substr(relname,1,%d)='%s' and pg_catalog.pg_table_is_visible(oid)"
+#define Query_for_list_of_indexes "SELECT relname FROM pg_catalog.pg_class WHERE relkind='i' and substr(relname,1,%d)='%s' and pg_catalog.pg_table_is_visible(oid)"
+#define Query_for_list_of_databases "SELECT datname FROM pg_catalog.pg_database WHERE substr(datname,1,%d)='%s'"
+#define Query_for_list_of_attributes "SELECT a.attname FROM pg_catalog.pg_attribute a, pg_catalog.pg_class c WHERE c.oid = a.attrelid and a.attnum>0 and not a.attisdropped and substr(a.attname,1,%d)='%s' and c.relname='%s' and pg_catalog.pg_table_is_visible(c.oid)"
+#define Query_for_list_of_users "SELECT usename FROM pg_catalog.pg_user WHERE substr(usename,1,%d)='%s'"
+
 /* This is a list of all "things" in Pgsql, which can show up after CREATE or
    DROP; and there is also a query to get a list of them.
-   The %s will be replaced by the text entered so far, the %d by its length.
-   If you change the order here or insert things, make sure to also adjust the
-   referencing macros below.
 */
 typedef struct
 {
@@ -132,37 +140,30 @@ typedef struct
 } pgsql_thing_t;
 
 pgsql_thing_t words_after_create[] = {
-	{"AGGREGATE", "SELECT distinct aggname FROM pg_aggregate WHERE substr(aggname,1,%d)='%s'"},
-	{"DATABASE", "SELECT datname FROM pg_database WHERE substr(datname,1,%d)='%s'"},
-	{"FUNCTION", "SELECT distinct proname FROM pg_proc WHERE substr(proname,1,%d)='%s'"},
-	{"GROUP", "SELECT groname FROM pg_group WHERE substr(groname,1,%d)='%s'"},
-	{"INDEX", "SELECT relname FROM pg_class WHERE relkind='i' and substr(relname,1,%d)='%s'"},
+	{"AGGREGATE", "SELECT DISTINCT proname FROM pg_catalog.pg_proc WHERE proisagg AND substr(proname,1,%d)='%s'"},
+	{"DATABASE", Query_for_list_of_databases},
+	{"FUNCTION", "SELECT DISTINCT proname FROM pg_catalog.pg_proc WHERE substr(proname,1,%d)='%s'"},
+	{"GROUP", "SELECT groname FROM pg_catalog.pg_group WHERE substr(groname,1,%d)='%s'"},
+	{"INDEX", Query_for_list_of_indexes},
 	{"OPERATOR", NULL},			/* Querying for this is probably not such
 								 * a good idea. */
-	{"RULE", "SELECT rulename FROM pg_rules WHERE substr(rulename,1,%d)='%s'"},
-	{"SEQUENCE", "SELECT relname FROM pg_class WHERE relkind='S' and substr(relname,1,%d)='%s'"},
-	{"TABLE", "SELECT relname FROM pg_class WHERE relkind='r' and substr(relname,1,%d)='%s'"},
+	{"RULE", "SELECT rulename FROM pg_catalog.pg_rules WHERE substr(rulename,1,%d)='%s'"},
+	{"SCHEMA", "SELECT nspname FROM pg_catalog.pg_namespace WHERE substr(nspname,1,%d)='%s'"},
+	{"SEQUENCE", "SELECT relname FROM pg_catalog.pg_class WHERE relkind='S' and substr(relname,1,%d)='%s'"},
+	{"TABLE", Query_for_list_of_tables},
 	{"TEMP", NULL},				/* for CREATE TEMP TABLE ... */
-	{"TRIGGER", "SELECT tgname FROM pg_trigger WHERE substr(tgname,1,%d)='%s'"},
-	{"TYPE", "SELECT typname FROM pg_type WHERE substr(typname,1,%d)='%s'"},
+	{"TRIGGER", "SELECT tgname FROM pg_catalog.pg_trigger WHERE substr(tgname,1,%d)='%s'"},
+	{"TYPE", "SELECT typname FROM pg_catalog.pg_type WHERE substr(typname,1,%d)='%s'"},
 	{"UNIQUE", NULL},			/* for CREATE UNIQUE INDEX ... */
-	{"USER", "SELECT usename FROM pg_user WHERE substr(usename,1,%d)='%s'"},
-	{"VIEW", "SELECT viewname FROM pg_views WHERE substr(viewname,1,%d)='%s'"},
+	{"USER", Query_for_list_of_users},
+	{"VIEW", "SELECT viewname FROM pg_catalog.pg_views WHERE substr(viewname,1,%d)='%s'"},
 	{NULL, NULL}				/* end of list */
 };
 
 
-/* The query to get a list of tables and a list of indexes, which are used at
-   various places. */
-#define Query_for_list_of_tables words_after_create[8].query
-#define Query_for_list_of_indexes words_after_create[4].query
-#define Query_for_list_of_databases words_after_create[1].query
-#define Query_for_list_of_attributes "SELECT a.attname FROM pg_attribute a, pg_class c WHERE c.oid = a.attrelid and a.attnum>0 and substr(a.attname,1,%d)='%s' and c.relname='%s'"
-#define Query_for_list_of_users words_after_create[13].query
-
 /* A couple of macros to ease typing. You can use these to complete the given
    string with
-   1) The results from a query you pass it. (Perhaps one of those right above?)
+   1) The results from a query you pass it. (Perhaps one of those above?)
    2) The items from a null-pointer-terminated list.
    3) A string constant
    4) The list of attributes to the given table.
@@ -197,7 +198,7 @@ psql_completion(char *text, int start, int end)
 	static char *sql_commands[] = {
 		"ABORT", "ALTER", "ANALYZE", "BEGIN", "CLOSE", "CLUSTER", "COMMENT", "COMMIT", "COPY",
 		"CREATE", "DECLARE", "DELETE", "DROP", "EXPLAIN", "FETCH", "GRANT",
-		"INSERT", "LISTEN", "LOAD", "LOCK", "MOVE", "NOTIFY", "RESET",
+		"INSERT", "LISTEN", "LOAD", "LOCK", "MOVE", "NOTIFY", "REINDEX", "RESET",
 		"REVOKE", "ROLLBACK", "SELECT", "SET", "SHOW", "TRUNCATE", "UNLISTEN", "UPDATE",
 		"VACUUM", NULL
 	};
@@ -226,11 +227,13 @@ psql_completion(char *text, int start, int end)
 		"enable_nestloop",
 		"enable_mergejoin",
 		"enable_hashjoin",
-		"ksqo",
 		"geqo",
 		"fsync",
+		"server_min_messages",
+		"client_min_messages",
 		"debug_assertions",
-		"debug_print_query",
+		"log_statement",
+		"log_duration",
 		"debug_print_parse",
 		"debug_print_rewritten",
 		"debug_print_plan",
@@ -238,13 +241,16 @@ psql_completion(char *text, int start, int end)
 		"show_parser_stats",
 		"show_planner_stats",
 		"show_executor_stats",
-		"show_query_stats",
+		"show_statement_stats",
 		"trace_notify",
+		"explain_pretty_print",
 		"sql_inheritance",
 		"australian_timezones",
 		"password_encryption",
 		"transform_null_equals",
+		"autocommit",
 
+		"default_statistics_target",
 		"geqo_threshold",
 		"geqo_pool_size",
 		"geqo_effort",
@@ -252,7 +258,6 @@ psql_completion(char *text, int start, int end)
 		"geqo_random_seed",
 		"sort_mem",
 		"vacuum_mem",
-		"debug_level",
 		"max_expr_depth",
 		"commit_delay",
 		"commit_siblings",
@@ -265,7 +270,9 @@ psql_completion(char *text, int start, int end)
 		"geqo_selection_bias",
 
 		"default_transaction_isolation",
-
+		"search_path",
+		"statement_timeout",
+		"log_min_error_statement",
 		NULL
 	};
 
@@ -275,8 +282,8 @@ psql_completion(char *text, int start, int end)
 		"\\e", "\\echo",
 		"\\encoding", "\\g", "\\h", "\\i", "\\l",
 		"\\lo_import", "\\lo_export", "\\lo_list", "\\lo_unlink",
-		"\\o", "\\p", "\\pset", "\\q", "\\qecho", "\\r", "\\set", "\\t", "\\unset",
-		"\\x", "\\w", "\\z", "\\!", NULL
+		"\\o", "\\p", "\\pset", "\\q", "\\qecho", "\\r", "\\set", "\\t",
+		"\\timing", "\\unset", "\\x", "\\w", "\\z", "\\!", NULL
 	};
 
 	(void) end;					/* not used */
@@ -318,7 +325,7 @@ psql_completion(char *text, int start, int end)
 	/* complete with what you can alter (TABLE, GROUP, USER) */
 	else if (strcasecmp(prev_wd, "ALTER") == 0)
 	{
-		char	   *list_ALTER[] = {"GROUP", "TABLE", "USER", NULL};
+		char	   *list_ALTER[] = {"GROUP", "SCHEMA", "TABLE", "USER", NULL};
 
 		COMPLETE_WITH_LIST(list_ALTER);
 	}
@@ -355,6 +362,14 @@ psql_completion(char *text, int start, int end)
 			 && strcasecmp(prev_wd, "USER") == 0)
 		COMPLETE_WITH_QUERY(Query_for_list_of_users);
 
+/* ANALYZE */
+	/* If the previous word is ANALYZE, produce list of tables. */
+	else if (strcasecmp(prev_wd, "ANALYZE") == 0)
+		COMPLETE_WITH_QUERY(Query_for_list_of_tables);
+	/* If we have ANALYZE <table>, complete with semicolon. */
+	else if (strcasecmp(prev2_wd, "ANALYZE") == 0)
+		COMPLETE_WITH_CONST(";");
+
 /* CLUSTER */
 	/* If the previous word is CLUSTER, produce list of indexes. */
 	else if (strcasecmp(prev_wd, "CLUSTER") == 0)
@@ -373,7 +388,7 @@ psql_completion(char *text, int start, int end)
 												 * queries. */
 
 		if (snprintf(query_buffer, BUF_SIZE,
-					 "SELECT c1.relname FROM pg_class c1, pg_class c2, pg_index i WHERE c1.oid=i.indrelid and i.indexrelid=c2.oid and c2.relname='%s'",
+					 "SELECT c1.relname FROM pg_catalog.pg_class c1, pg_catalog.pg_class c2, pg_catalog.pg_index i WHERE c1.oid=i.indrelid and i.indexrelid=c2.oid and c2.relname='%s' and pg_catalog.pg_table_is_visible(c2.oid)",
 					 prev2_wd) == -1)
 			ERROR_QUERY_TOO_LONG;
 		else
@@ -386,8 +401,9 @@ psql_completion(char *text, int start, int end)
 	else if (strcasecmp(prev2_wd, "COMMENT") == 0 && strcasecmp(prev_wd, "ON") == 0)
 	{
 		char	   *list_COMMENT[] =
-		{"DATABASE", "INDEX", "RULE", "SEQUENCE", "TABLE", "TYPE", "VIEW",
-		"COLUMN", "AGGREGATE", "FUNCTION", "OPERATOR", "TRIGGER", NULL};
+		{"DATABASE", "INDEX", "RULE", "SCHEMA", "SEQUENCE", "TABLE", "TYPE", "VIEW",
+			"COLUMN", "AGGREGATE", "FUNCTION", "OPERATOR", "TRIGGER", "CONSTRAINT",
+		"DOMAIN", NULL};
 
 		COMPLETE_WITH_LIST(list_COMMENT);
 	}
@@ -430,7 +446,7 @@ psql_completion(char *text, int start, int end)
 	 * Complete INDEX <name> ON <table> with a list of table columns
 	 * (which should really be in parens)
 	 */
-	else if ((strcasecmp(prev4_wd, "INDEX") == 0 && strcasecmp(prev2_wd, "ON") == 0))
+	else if (strcasecmp(prev4_wd, "INDEX") == 0 && strcasecmp(prev2_wd, "ON") == 0)
 		COMPLETE_WITH_ATTR(prev_wd);
 	/* same if you put in USING */
 	else if ((strcasecmp(prev4_wd, "ON") == 0 && strcasecmp(prev2_wd, "USING") == 0))
@@ -438,7 +454,7 @@ psql_completion(char *text, int start, int end)
 	/* Complete USING with an index method */
 	else if (strcasecmp(prev_wd, "USING") == 0)
 	{
-		char	   *index_mth[] = {"BTREE", "RTREE", "HASH", NULL};
+		char	   *index_mth[] = {"BTREE", "RTREE", "HASH", "GIST", NULL};
 
 		COMPLETE_WITH_LIST(index_mth);
 	}
@@ -495,10 +511,11 @@ psql_completion(char *text, int start, int end)
 
 	/*
 	 * Complete DELETE with FROM (only if the word before that is not "ON"
-	 * (cf. rules) or "BEFORE" or "AFTER" (cf. triggers) )
+	 * (cf. rules) or "BEFORE" or "AFTER" (cf. triggers) or GRANT)
 	 */
 	else if (strcasecmp(prev_wd, "DELETE") == 0 &&
 			 !(strcasecmp(prev2_wd, "ON") == 0 ||
+			   strcasecmp(prev2_wd, "GRANT") == 0 ||
 			   strcasecmp(prev2_wd, "BEFORE") == 0 ||
 			   strcasecmp(prev2_wd, "AFTER") == 0))
 		COMPLETE_WITH_CONST("FROM");
@@ -550,7 +567,7 @@ psql_completion(char *text, int start, int end)
 	/* Complete GRANT/REVOKE with a list of privileges */
 	else if (strcasecmp(prev_wd, "GRANT") == 0 || strcasecmp(prev_wd, "REVOKE") == 0)
 	{
-		char	   *list_privileg[] = {"SELECT", "INSERT", "UPDATE", "DELETE", "RULE", "ALL", NULL};
+		char	   *list_privileg[] = {"SELECT", "INSERT", "UPDATE", "DELETE", "RULE", "REFERENCES", "TRIGGER", "CREATE", "TEMPORARY", "EXECUTE", "USAGE", "ALL", NULL};
 
 		COMPLETE_WITH_LIST(list_privileg);
 	}
@@ -561,10 +578,14 @@ psql_completion(char *text, int start, int end)
 	/*
 	 * Complete GRANT/REVOKE <sth> ON with a list of tables, views,
 	 * sequences, and indexes
+	 *
+	 * XXX should also offer DATABASE, FUNCTION, LANGUAGE, SCHEMA here
 	 */
 	else if ((strcasecmp(prev3_wd, "GRANT") == 0 || strcasecmp(prev3_wd, "REVOKE") == 0) &&
 			 strcasecmp(prev_wd, "ON") == 0)
-		COMPLETE_WITH_QUERY("SELECT relname FROM pg_class WHERE relkind in ('r','i','S','v') and substr(relname,1,%d)='%s'");
+		COMPLETE_WITH_QUERY("SELECT relname FROM pg_catalog.pg_class "
+							"WHERE relkind in ('r','i','S','v') AND "
+							"substr(relname,1,%d)='%s' AND pg_catalog.pg_table_is_visible(oid)");
 	/* Complete "GRANT * ON * " with "TO" */
 	else if (strcasecmp(prev4_wd, "GRANT") == 0 && strcasecmp(prev2_wd, "ON") == 0)
 		COMPLETE_WITH_CONST("TO");
@@ -584,6 +605,9 @@ psql_completion(char *text, int start, int end)
 	/* Complete INSERT INTO with table names */
 	else if (strcasecmp(prev2_wd, "INSERT") == 0 && strcasecmp(prev_wd, "INTO") == 0)
 		COMPLETE_WITH_QUERY(Query_for_list_of_tables);
+	/* Complete "INSERT INTO <table> (" with attribute names */
+	else if (rl_line_buffer[start - 1] == '(' && strcasecmp(prev3_wd, "INSERT") == 0 && strcasecmp(prev2_wd, "INTO") == 0)
+		COMPLETE_WITH_ATTR(prev_wd);
 
 	/*
 	 * Complete INSERT INTO <table> with "VALUES" or "SELECT" or "DEFAULT
@@ -595,19 +619,47 @@ psql_completion(char *text, int start, int end)
 
 		COMPLETE_WITH_LIST(list_INSERT);
 	}
+	/* Complete INSERT INTO <table> (attribs) with "VALUES" or "SELECT" */
+	else if (strcasecmp(prev4_wd, "INSERT") == 0 && strcasecmp(prev3_wd, "INTO") == 0 &&
+			 prev_wd[strlen(prev_wd) - 1] == ')')
+	{
+		char	   *list_INSERT[] = {"SELECT", "VALUES", NULL};
+
+		COMPLETE_WITH_LIST(list_INSERT);
+	}
+
 	/* Insert an open parenthesis after "VALUES" */
 	else if (strcasecmp(prev_wd, "VALUES") == 0 && strcasecmp(prev2_wd, "DEFAULT") != 0)
 		COMPLETE_WITH_CONST("(");
 
 /* LOCK */
-	/* Complete with list of tables */
-	else if (strcasecmp(prev_wd, "LOCK") == 0)
+	/* Complete LOCK [TABLE] with a list of tables */
+	else if ((strcasecmp(prev_wd, "LOCK") == 0) ||
+	 (strcasecmp(prev_wd, "TABLE") == 0 && strcasecmp(prev2_wd, "LOCK")))
 		COMPLETE_WITH_QUERY(Query_for_list_of_tables);
-	/* (If you want more with LOCK, you better think about it yourself.) */
+
+	/* For the following, handle the case of a single table only for now */
+
+	/* Complete LOCK [TABLE] <table> with "IN" */
+	else if ((strcasecmp(prev2_wd, "LOCK") == 0 && strcasecmp(prev_wd, "TABLE")) ||
+			 (strcasecmp(prev2_wd, "TABLE") == 0 && strcasecmp(prev3_wd, "LOCK") == 0))
+		COMPLETE_WITH_CONST("IN");
+
+	/* Complete LOCK [TABLE] <table> IN with a lock mode */
+	else if (strcasecmp(prev_wd, "IN") == 0 &&
+			 (strcasecmp(prev3_wd, "LOCK") == 0 ||
+	(strcasecmp(prev3_wd, "TABLE") == 0 && strcasecmp(prev3_wd, "LOCK"))))
+	{
+		char	   *lock_modes[] = {"ACCESS SHARE MODE", "ROW SHARE MODE", "ROW EXCLUSIVE MODE",
+			"SHARE UPDATE EXCLUSIVE MODE", "SHARE MODE", "SHARE ROW EXCLUSIVE MODE",
+		"EXCLUSIVE MODE", "ACCESS EXCLUSIVE MODE", NULL};
+
+		COMPLETE_WITH_LIST(lock_modes);
+	}
 
 /* NOTIFY */
 	else if (strcasecmp(prev_wd, "NOTIFY") == 0)
-		COMPLETE_WITH_QUERY("SELECT relname FROM pg_listener WHERE substr(relname,1,%d)='%s'");
+		COMPLETE_WITH_QUERY("SELECT relname FROM pg_catalog.pg_listener WHERE substr(relname,1,%d)='%s'");
 
 /* REINDEX */
 	else if (strcasecmp(prev_wd, "REINDEX") == 0)
@@ -691,7 +743,7 @@ psql_completion(char *text, int start, int end)
 
 			COMPLETE_WITH_LIST(my_list);
 		}
-		else if (strcasecmp(prev2_wd, "GEQO") == 0 || strcasecmp(prev2_wd, "KSQO") == 0)
+		else if (strcasecmp(prev2_wd, "GEQO") == 0)
 		{
 			char	   *my_list[] = {"ON", "OFF", "DEFAULT", NULL};
 
@@ -711,7 +763,7 @@ psql_completion(char *text, int start, int end)
 
 /* UNLISTEN */
 	else if (strcasecmp(prev_wd, "UNLISTEN") == 0)
-		COMPLETE_WITH_QUERY("SELECT relname FROM pg_listener WHERE substr(relname,1,%d)='%s' UNION SELECT '*'::text");
+		COMPLETE_WITH_QUERY("SELECT relname FROM pg_catalog.pg_listener WHERE substr(relname,1,%d)='%s' UNION SELECT '*'::name");
 
 /* UPDATE */
 	/* If prev. word is UPDATE suggest a list of tables */
@@ -731,10 +783,14 @@ psql_completion(char *text, int start, int end)
 
 /* VACUUM */
 	else if (strcasecmp(prev_wd, "VACUUM") == 0)
-		COMPLETE_WITH_QUERY("SELECT relname FROM pg_class WHERE relkind='r' and substr(relname,1,%d)='%s' UNION SELECT 'ANALYZE'::text");
-	else if (strcasecmp(prev2_wd, "VACUUM") == 0 && strcasecmp(prev_wd, "ANALYZE") == 0)
+		COMPLETE_WITH_QUERY("SELECT relname FROM pg_catalog.pg_class WHERE relkind='r' and substr(relname,1,%d)='%s' and pg_catalog.pg_table_is_visible(oid) UNION SELECT 'FULL'::name UNION SELECT 'ANALYZE'::name");
+	else if (strcasecmp(prev2_wd, "VACUUM") == 0 && (strcasecmp(prev_wd, "FULL") == 0 || strcasecmp(prev_wd, "ANALYZE") == 0))
 		COMPLETE_WITH_QUERY(Query_for_list_of_tables);
 
+/* WHERE */
+	/* Simple case of the word before the where being the table name */
+	else if (strcasecmp(prev_wd, "WHERE") == 0)
+		COMPLETE_WITH_ATTR(prev2_wd);
 
 /* ... FROM ... */
 	else if (strcasecmp(prev_wd, "FROM") == 0)
@@ -1005,15 +1061,14 @@ exec_query(char *query)
 
 /* Return the word (space delimited) before point. Set skip > 0 to skip that
    many words; e.g. skip=1 finds the word before the previous one.
-   TODO: Take account of quotes. (Right now, if you table names contain spaces
-   you're screwed.)
 */
 static char *
 previous_word(int point, int skip)
 {
 	int			i,
 				start = 0,
-				end = -1;
+				end = -1,
+				inquotes = 0;
 	char	   *s;
 
 	while (skip-- >= 0)
@@ -1044,8 +1099,12 @@ previous_word(int point, int skip)
 		 * or it's simply character 0
 		 */
 		for (start = end; start > 0; start--)
-			if (rl_line_buffer[start - 1] == ' ')
+		{
+			if (rl_line_buffer[start] == '"')
+				inquotes = !inquotes;
+			if ((rl_line_buffer[start - 1] == ' ') && inquotes == 0)
 				break;
+		}
 
 		point = start;
 	}
