@@ -3,7 +3,7 @@
  *			  procedural language
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/pl/plpgsql/src/pl_handler.c,v 1.10 2001/10/25 05:50:20 momjian Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/pl/plpgsql/src/pl_handler.c,v 1.12 2002/08/30 00:28:41 tgl Exp $
  *
  *	  This software is copyrighted by Jan Wieck - Hamburg.
  *
@@ -35,13 +35,6 @@
  *
  **********************************************************************/
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <string.h>
-
 #include "plpgsql.h"
 #include "pl.tab.h"
 
@@ -167,8 +160,8 @@ func_up_to_date(PLpgSQL_function * func)
 		elog(ERROR, "plpgsql: cache lookup for proc %u failed",
 			 func->fn_oid);
 
-	result = (func->fn_xmin == procTup->t_data->t_xmin &&
-			  func->fn_cmin == procTup->t_data->t_cmin);
+	result = (func->fn_xmin == HeapTupleHeaderGetXmin(procTup->t_data) &&
+			  func->fn_cmin == HeapTupleHeaderGetCmin(procTup->t_data));
 
 	ReleaseSysCache(procTup);
 
