@@ -31,7 +31,7 @@
  *	  ENHANCEMENTS, OR MODIFICATIONS.
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/pl/tcl/pltcl.c,v 1.49 2002/01/24 19:31:36 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/pl/tcl/pltcl.c,v 1.66.2.3 2003/05/16 13:37:11 tgl Exp $
  *
  **********************************************************************/
 
@@ -39,45 +39,38 @@
 
 #include <tcl.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <string.h>
 #include <setjmp.h>
 
-#include "executor/spi.h"
-#include "commands/trigger.h"
-#include "utils/builtins.h"
-#include "fmgr.h"
 #include "access/heapam.h"
-
-#include "tcop/tcopprot.h"
-#include "utils/syscache.h"
-#include "catalog/pg_proc.h"
 #include "catalog/pg_language.h"
+#include "catalog/pg_proc.h"
 #include "catalog/pg_type.h"
+#include "commands/trigger.h"
+#include "executor/spi.h"
+#include "fmgr.h"
+#include "nodes/makefuncs.h"
+#include "parser/parse_type.h"
+#include "tcop/tcopprot.h"
+#include "utils/builtins.h"
+#include "utils/syscache.h"
 
 #if defined(UNICODE_CONVERSION) && TCL_MAJOR_VERSION == 8 \
 	&& TCL_MINOR_VERSION > 0
 
 #include "mb/pg_wchar.h"
 
-static pg_enconv *tcl_enconv;
-
 static unsigned char *
 utf_u2e(unsigned char *src)
 {
-	return pg_do_encoding_conversion(src, strlen(src),
-									 NULL, tcl_enconv->from_unicode);
+	return pg_do_encoding_conversion(src, strlen(src), PG_UTF8, GetDatabaseEncoding());
 }
 
 static unsigned char *
 utf_e2u(unsigned char *src)
 {
-	return pg_do_encoding_conversion(src, strlen(src),
-									 tcl_enconv->to_unicode, NULL);
+	return pg_do_encoding_conversion(src, strlen(src), GetDatabaseEncoding(), PG_UTF8);
 }
 
 #define PLTCL_UTF
@@ -210,14 +203,6 @@ pltcl_init_all(void)
 	if (!pltcl_firstcall)
 		return;
 
-#ifdef PLTCL_UTF
-	/************************************************************
-	 * Do unicode conversion initialization
-	 ************************************************************/
-
-	tcl_enconv = pg_get_enconv_by_encoding(GetDatabaseEncoding());
-#endif
-
 	/************************************************************
 	 * Create the dummy hold interpreter to prevent close of
 	 * stdout and stderr on DeleteInterp
@@ -317,8 +302,9 @@ pltcl_init_load_unknown(Tcl_Interp *interp)
 	/************************************************************
 	 * Check if table pltcl_modules exists
 	 ************************************************************/
-	spi_rc = SPI_exec("select 1 from pg_class "
+	spi_rc = SPI_exec("select 1 from pg_catalog.pg_class "
 					  "where relname = 'pltcl_modules'", 1);
+	SPI_freetuptable(SPI_tuptable);
 	if (spi_rc != SPI_OK_SELECT)
 		elog(ERROR, "pltcl_init_load_unknown(): select from pg_class failed");
 	if (SPI_processed == 0)
@@ -345,7 +331,8 @@ pltcl_init_load_unknown(Tcl_Interp *interp)
 	if (SPI_processed == 0)
 	{
 		Tcl_DStringFree(&unknown_src);
-		elog(NOTICE, "pltcl: Module unknown not found in pltcl_modules");
+		SPI_freetuptable(SPI_tuptable);
+		elog(WARNING, "pltcl: Module unknown not found in pltcl_modules");
 		return;
 	}
 
@@ -370,6 +357,7 @@ pltcl_init_load_unknown(Tcl_Interp *interp)
 	}
 	tcl_rc = Tcl_GlobalEval(interp, Tcl_DStringValue(&unknown_src));
 	Tcl_DStringFree(&unknown_src);
+	SPI_freetuptable(SPI_tuptable);
 }
 
 
@@ -593,15 +581,17 @@ pltcl_func_handler(PG_FUNCTION_ARGS)
 	if (SPI_finish() != SPI_OK_FINISH)
 		elog(ERROR, "pltcl: SPI_finish() failed");
 
-	UTF_BEGIN;
 	if (fcinfo->isnull)
 		retval = (Datum) 0;
 	else
+	{
+		UTF_BEGIN;
 		retval = FunctionCall3(&prodesc->result_in_func,
 							   PointerGetDatum(UTF_U2E(interp->result)),
 							   ObjectIdGetDatum(prodesc->result_in_elem),
 							   Int32GetDatum(-1));
-	UTF_END;
+		UTF_END;
+	}
 
 	/************************************************************
 	 * Finally we may restore normal error handling.
@@ -826,7 +816,7 @@ pltcl_trigger_handler(PG_FUNCTION_ARGS)
 	if (Tcl_SplitList(interp, interp->result,
 					  &ret_numvals, &ret_values) != TCL_OK)
 	{
-		elog(NOTICE, "pltcl: cannot split return value from trigger");
+		elog(WARNING, "pltcl: cannot split return value from trigger");
 		elog(ERROR, "pltcl: %s", interp->result);
 	}
 
@@ -964,9 +954,11 @@ compile_pltcl_function(Oid fn_oid, bool is_trigger)
 	 * Build our internal proc name from the functions Oid
 	 ************************************************************/
 	if (!is_trigger)
-		sprintf(internal_proname, "__PLTcl_proc_%u", fn_oid);
+		snprintf(internal_proname, sizeof(internal_proname),
+				 "__PLTcl_proc_%u", fn_oid);
 	else
-		sprintf(internal_proname, "__PLTcl_proc_%u_trigger", fn_oid);
+		snprintf(internal_proname, sizeof(internal_proname),
+				 "__PLTcl_proc_%u_trigger", fn_oid);
 
 	/************************************************************
 	 * Lookup the internal proc name in the hashtable
@@ -984,8 +976,8 @@ compile_pltcl_function(Oid fn_oid, bool is_trigger)
 
 		prodesc = (pltcl_proc_desc *) Tcl_GetHashValue(hashent);
 
-		uptodate = (prodesc->fn_xmin == procTup->t_data->t_xmin &&
-					prodesc->fn_cmin == procTup->t_data->t_cmin);
+		uptodate = (prodesc->fn_xmin == HeapTupleHeaderGetXmin(procTup->t_data) &&
+			prodesc->fn_cmin == HeapTupleHeaderGetCmin(procTup->t_data));
 
 		if (!uptodate)
 		{
@@ -1022,8 +1014,8 @@ compile_pltcl_function(Oid fn_oid, bool is_trigger)
 			elog(ERROR, "pltcl: out of memory");
 		MemSet(prodesc, 0, sizeof(pltcl_proc_desc));
 		prodesc->proname = strdup(internal_proname);
-		prodesc->fn_xmin = procTup->t_data->t_xmin;
-		prodesc->fn_cmin = procTup->t_data->t_cmin;
+		prodesc->fn_xmin = HeapTupleHeaderGetXmin(procTup->t_data);
+		prodesc->fn_cmin = HeapTupleHeaderGetCmin(procTup->t_data);
 
 		/************************************************************
 		 * Lookup the pg_language tuple by Oid
@@ -1060,15 +1052,33 @@ compile_pltcl_function(Oid fn_oid, bool is_trigger)
 			{
 				free(prodesc->proname);
 				free(prodesc);
-				if (!OidIsValid(procStruct->prorettype))
-					elog(ERROR, "pltcl functions cannot return type \"opaque\""
-						 "\n\texcept when used as triggers");
-				else
-					elog(ERROR, "pltcl: cache lookup for return type %u failed",
-						 procStruct->prorettype);
+				elog(ERROR, "pltcl: cache lookup for return type %u failed",
+					 procStruct->prorettype);
 			}
 			typeStruct = (Form_pg_type) GETSTRUCT(typeTup);
 
+			/* Disallow pseudotype result, except VOID */
+			if (typeStruct->typtype == 'p')
+			{
+				if (procStruct->prorettype == VOIDOID)
+					 /* okay */ ;
+				else if (procStruct->prorettype == TRIGGEROID)
+				{
+					free(prodesc->proname);
+					free(prodesc);
+					elog(ERROR, "pltcl functions cannot return type %s"
+						 "\n\texcept when used as triggers",
+						 format_type_be(procStruct->prorettype));
+				}
+				else
+				{
+					free(prodesc->proname);
+					free(prodesc);
+					elog(ERROR, "pltcl functions cannot return type %s",
+						 format_type_be(procStruct->prorettype));
+				}
+			}
+
 			if (typeStruct->typrelid != InvalidOid)
 			{
 				free(prodesc->proname);
@@ -1099,20 +1109,26 @@ compile_pltcl_function(Oid fn_oid, bool is_trigger)
 				{
 					free(prodesc->proname);
 					free(prodesc);
-					if (!OidIsValid(procStruct->proargtypes[i]))
-						elog(ERROR, "pltcl functions cannot take type \"opaque\"");
-					else
-						elog(ERROR, "pltcl: cache lookup for argument type %u failed",
-							 procStruct->proargtypes[i]);
+					elog(ERROR, "pltcl: cache lookup for argument type %u failed",
+						 procStruct->proargtypes[i]);
 				}
 				typeStruct = (Form_pg_type) GETSTRUCT(typeTup);
 
+				/* Disallow pseudotype argument */
+				if (typeStruct->typtype == 'p')
+				{
+					free(prodesc->proname);
+					free(prodesc);
+					elog(ERROR, "pltcl functions cannot take type %s",
+						 format_type_be(procStruct->proargtypes[i]));
+				}
+
 				if (typeStruct->typrelid != InvalidOid)
 				{
 					prodesc->arg_is_rel[i] = 1;
 					if (i > 0)
 						strcat(proc_internal_args, " ");
-					sprintf(buf, "__PLTcl_Tup_%d", i + 1);
+					snprintf(buf, sizeof(buf), "__PLTcl_Tup_%d", i + 1);
 					strcat(proc_internal_args, buf);
 					ReleaseSysCache(typeTup);
 					continue;
@@ -1125,7 +1141,7 @@ compile_pltcl_function(Oid fn_oid, bool is_trigger)
 
 				if (i > 0)
 					strcat(proc_internal_args, " ");
-				sprintf(buf, "%d", i + 1);
+				snprintf(buf, sizeof(buf), "%d", i + 1);
 				strcat(proc_internal_args, buf);
 
 				ReleaseSysCache(typeTup);
@@ -1162,7 +1178,8 @@ compile_pltcl_function(Oid fn_oid, bool is_trigger)
 			{
 				if (!prodesc->arg_is_rel[i])
 					continue;
-				sprintf(buf, "array set %d $__PLTcl_Tup_%d\n", i + 1, i + 1);
+				snprintf(buf, sizeof(buf), "array set %d $__PLTcl_Tup_%d\n",
+						 i + 1, i + 1);
 				Tcl_DStringAppend(&proc_internal_body, buf, -1);
 			}
 		}
@@ -1231,7 +1248,7 @@ static int
 pltcl_elog(ClientData cdata, Tcl_Interp *interp,
 		   int argc, char *argv[])
 {
-	int			level;
+	volatile int level;
 	sigjmp_buf	save_restart;
 
 	/************************************************************
@@ -1240,8 +1257,36 @@ pltcl_elog(ClientData cdata, Tcl_Interp *interp,
 	if (pltcl_restart_in_progress)
 		return TCL_ERROR;
 
+	if (argc != 3)
+	{
+		Tcl_SetResult(interp, "syntax error - 'elog level msg'",
+					  TCL_VOLATILE);
+		return TCL_ERROR;
+	}
+
+	if (strcmp(argv[1], "DEBUG") == 0)
+		level = DEBUG1;
+	else if (strcmp(argv[1], "LOG") == 0)
+		level = LOG;
+	else if (strcmp(argv[1], "INFO") == 0)
+		level = INFO;
+	else if (strcmp(argv[1], "NOTICE") == 0)
+		level = NOTICE;
+	else if (strcmp(argv[1], "WARNING") == 0)
+		level = WARNING;
+	else if (strcmp(argv[1], "ERROR") == 0)
+		level = ERROR;
+	else if (strcmp(argv[1], "FATAL") == 0)
+		level = FATAL;
+	else
+	{
+		Tcl_AppendResult(interp, "Unknown elog level '", argv[1],
+						 "'", NULL);
+		return TCL_ERROR;
+	}
+
 	/************************************************************
-	 * Catch the restart longjmp and begin a controlled
+	 * Catch the longjmp from elog() and begin a controlled
 	 * return though all interpreter levels if it happens
 	 ************************************************************/
 	memcpy(&save_restart, &Warn_restart, sizeof(save_restart));
@@ -1252,38 +1297,14 @@ pltcl_elog(ClientData cdata, Tcl_Interp *interp,
 		return TCL_ERROR;
 	}
 
-	if (argc != 3)
-	{
-		Tcl_SetResult(interp, "syntax error - 'elog level msg'",
-					  TCL_VOLATILE);
-		return TCL_ERROR;
-	}
-
-	if (strcmp(argv[1], "NOTICE") == 0)
-		level = NOTICE;
-	else if (strcmp(argv[1], "WARN") == 0)
-		level = ERROR;
-	else if (strcmp(argv[1], "ERROR") == 0)
-		level = ERROR;
-	else if (strcmp(argv[1], "FATAL") == 0)
-		level = FATAL;
-	else if (strcmp(argv[1], "DEBUG") == 0)
-		level = DEBUG;
-	else
-	{
-		Tcl_AppendResult(interp, "Unknown elog level '", argv[1],
-						 "'", NULL);
-		memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
-		return TCL_ERROR;
-	}
-
 	/************************************************************
 	 * Call elog(), restore the original restart address
-	 * and return to the caller (if not catched)
+	 * and return to the caller (if no longjmp)
 	 ************************************************************/
 	UTF_BEGIN;
-	elog(level, UTF_U2E(argv[2]));
+	elog(level, "%s", UTF_U2E(argv[2]));
 	UTF_END;
+
 	memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
 	return TCL_OK;
 }
@@ -1456,6 +1477,7 @@ pltcl_SPI_exec(ClientData cdata, Tcl_Interp *interp,
 	int			ntuples;
 	HeapTuple  *volatile tuples;
 	volatile TupleDesc tupdesc = NULL;
+	SPITupleTable *tuptable;
 	sigjmp_buf	save_restart;
 
 	char	   *usage = "syntax error - 'SPI_exec "
@@ -1538,14 +1560,16 @@ pltcl_SPI_exec(ClientData cdata, Tcl_Interp *interp,
 	{
 		case SPI_OK_UTILITY:
 			Tcl_SetResult(interp, "0", TCL_VOLATILE);
+			SPI_freetuptable(SPI_tuptable);
 			return TCL_OK;
 
 		case SPI_OK_SELINTO:
 		case SPI_OK_INSERT:
 		case SPI_OK_DELETE:
 		case SPI_OK_UPDATE:
-			sprintf(buf, "%d", SPI_processed);
+			snprintf(buf, sizeof(buf), "%d", SPI_processed);
 			Tcl_SetResult(interp, buf, TCL_VOLATILE);
+			SPI_freetuptable(SPI_tuptable);
 			return TCL_OK;
 
 		case SPI_OK_SELECT:
@@ -1588,7 +1612,7 @@ pltcl_SPI_exec(ClientData cdata, Tcl_Interp *interp,
 			return TCL_ERROR;
 
 		default:
-			sprintf(buf, "%d", spi_rc);
+			snprintf(buf, sizeof(buf), "%d", spi_rc);
 			Tcl_AppendResult(interp, "pltcl: SPI_exec() failed - ",
 							 "unknown RC ", buf, NULL);
 			return TCL_ERROR;
@@ -1626,12 +1650,15 @@ pltcl_SPI_exec(ClientData cdata, Tcl_Interp *interp,
 	{
 		if (ntuples > 0)
 			pltcl_set_tuple_values(interp, arrayname, 0, tuples[0], tupdesc);
-		sprintf(buf, "%d", ntuples);
+		snprintf(buf, sizeof(buf), "%d", ntuples);
 		Tcl_SetResult(interp, buf, TCL_VOLATILE);
+		SPI_freetuptable(SPI_tuptable);
 		memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
 		return TCL_OK;
 	}
 
+	tuptable = SPI_tuptable;
+
 	/************************************************************
 	 * There is a loop body - process all tuples and evaluate
 	 * the body on each
@@ -1649,20 +1676,24 @@ pltcl_SPI_exec(ClientData cdata, Tcl_Interp *interp,
 			continue;
 		if (loop_rc == TCL_RETURN)
 		{
+			SPI_freetuptable(tuptable);
 			memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
 			return TCL_RETURN;
 		}
 		if (loop_rc == TCL_BREAK)
 			break;
+		SPI_freetuptable(tuptable);
 		memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
 		return TCL_ERROR;
 	}
 
+	SPI_freetuptable(tuptable);
+
 	/************************************************************
 	 * Finally return the number of tuples
 	 ************************************************************/
 	memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
-	sprintf(buf, "%d", ntuples);
+	snprintf(buf, sizeof(buf), "%d", ntuples);
 	Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	return TCL_OK;
 }
@@ -1671,7 +1702,7 @@ pltcl_SPI_exec(ClientData cdata, Tcl_Interp *interp,
 /**********************************************************************
  * pltcl_SPI_prepare()		- Builtin support for prepared plans
  *				  The Tcl command SPI_prepare
- *				  allways saves the plan using
+ *				  always saves the plan using
  *				  SPI_saveplan and returns a key for
  *				  access. There is no chance to prepare
  *				  and not save the plan currently.
@@ -1717,7 +1748,7 @@ pltcl_SPI_prepare(ClientData cdata, Tcl_Interp *interp,
 	 * Allocate the new querydesc structure
 	 ************************************************************/
 	qdesc = (pltcl_query_desc *) malloc(sizeof(pltcl_query_desc));
-	sprintf(qdesc->qname, "%lx", (long) qdesc);
+	snprintf(qdesc->qname, sizeof(qdesc->qname), "%lx", (long) qdesc);
 	qdesc->nargs = nargs;
 	qdesc->argtypes = (Oid *) malloc(nargs * sizeof(Oid));
 	qdesc->arginfuncs = (FmgrInfo *) malloc(nargs * sizeof(FmgrInfo));
@@ -1746,12 +1777,9 @@ pltcl_SPI_prepare(ClientData cdata, Tcl_Interp *interp,
 	 ************************************************************/
 	for (i = 0; i < nargs; i++)
 	{
-		typeTup = SearchSysCache(TYPENAME,
-								 PointerGetDatum(args[i]),
-								 0, 0, 0);
-		if (!HeapTupleIsValid(typeTup))
-			elog(ERROR, "pltcl: Cache lookup of type %s failed", args[i]);
-		qdesc->argtypes[i] = typeTup->t_data->t_oid;
+		/* XXX should extend this to allow qualified type names */
+		typeTup = typenameType(makeTypeName(args[i]));
+		qdesc->argtypes[i] = HeapTupleGetOid(typeTup);
 		perm_fmgr_info(((Form_pg_type) GETSTRUCT(typeTup))->typinput,
 					   &(qdesc->arginfuncs[i]));
 		qdesc->argtypelems[i] = ((Form_pg_type) GETSTRUCT(typeTup))->typelem;
@@ -1799,7 +1827,7 @@ pltcl_SPI_prepare(ClientData cdata, Tcl_Interp *interp,
 				break;
 
 			default:
-				sprintf(buf, "unknown RC %d", SPI_result);
+				snprintf(buf, sizeof(buf), "unknown RC %d", SPI_result);
 				reason = buf;
 				break;
 
@@ -1809,7 +1837,8 @@ pltcl_SPI_prepare(ClientData cdata, Tcl_Interp *interp,
 	}
 
 	/************************************************************
-	 * Save the plan
+	 * Save the plan into permanent memory (right now it's in the
+	 * SPI procCxt, which will go away at function end).
 	 ************************************************************/
 	qdesc->plan = SPI_saveplan(plan);
 	if (qdesc->plan == NULL)
@@ -1830,7 +1859,7 @@ pltcl_SPI_prepare(ClientData cdata, Tcl_Interp *interp,
 				break;
 
 			default:
-				sprintf(buf, "unknown RC %d", SPI_result);
+				snprintf(buf, sizeof(buf), "unknown RC %d", SPI_result);
 				reason = buf;
 				break;
 
@@ -1838,6 +1867,8 @@ pltcl_SPI_prepare(ClientData cdata, Tcl_Interp *interp,
 
 		elog(ERROR, "pltcl: SPI_saveplan() failed - %s", reason);
 	}
+	/* Release the procCxt copy to avoid within-function memory leak */
+	SPI_freeplan(plan);
 
 	/************************************************************
 	 * Insert a hashtable entry for the plan and return
@@ -1871,7 +1902,7 @@ pltcl_SPI_execp(ClientData cdata, Tcl_Interp *interp,
 	int			loop_body;
 	Tcl_HashEntry *hashent;
 	pltcl_query_desc *qdesc;
-	Datum	   *argvalues = NULL;
+	Datum	   *volatile argvalues = NULL;
 	char	   *volatile nulls = NULL;
 	char	   *volatile arrayname = NULL;
 	int			count = 0;
@@ -1881,6 +1912,7 @@ pltcl_SPI_execp(ClientData cdata, Tcl_Interp *interp,
 	int			ntuples;
 	HeapTuple  *volatile tuples = NULL;
 	volatile TupleDesc tupdesc = NULL;
+	SPITupleTable *tuptable;
 	sigjmp_buf	save_restart;
 	Tcl_HashTable *query_hash;
 
@@ -2100,14 +2132,16 @@ pltcl_SPI_execp(ClientData cdata, Tcl_Interp *interp,
 	{
 		case SPI_OK_UTILITY:
 			Tcl_SetResult(interp, "0", TCL_VOLATILE);
+			SPI_freetuptable(SPI_tuptable);
 			return TCL_OK;
 
 		case SPI_OK_SELINTO:
 		case SPI_OK_INSERT:
 		case SPI_OK_DELETE:
 		case SPI_OK_UPDATE:
-			sprintf(buf, "%d", SPI_processed);
+			snprintf(buf, sizeof(buf), "%d", SPI_processed);
 			Tcl_SetResult(interp, buf, TCL_VOLATILE);
+			SPI_freetuptable(SPI_tuptable);
 			return TCL_OK;
 
 		case SPI_OK_SELECT:
@@ -2150,7 +2184,7 @@ pltcl_SPI_execp(ClientData cdata, Tcl_Interp *interp,
 			return TCL_ERROR;
 
 		default:
-			sprintf(buf, "%d", spi_rc);
+			snprintf(buf, sizeof(buf), "%d", spi_rc);
 			Tcl_AppendResult(interp, "pltcl: SPI_exec() failed - ",
 							 "unknown RC ", buf, NULL);
 			return TCL_ERROR;
@@ -2192,11 +2226,14 @@ pltcl_SPI_execp(ClientData cdata, Tcl_Interp *interp,
 		if (ntuples > 0)
 			pltcl_set_tuple_values(interp, arrayname, 0, tuples[0], tupdesc);
 		memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
-		sprintf(buf, "%d", ntuples);
+		snprintf(buf, sizeof(buf), "%d", ntuples);
 		Tcl_SetResult(interp, buf, TCL_VOLATILE);
+		SPI_freetuptable(SPI_tuptable);
 		return TCL_OK;
 	}
 
+	tuptable = SPI_tuptable;
+
 	/************************************************************
 	 * There is a loop body - process all tuples and evaluate
 	 * the body on each
@@ -2213,20 +2250,24 @@ pltcl_SPI_execp(ClientData cdata, Tcl_Interp *interp,
 			continue;
 		if (loop_rc == TCL_RETURN)
 		{
+			SPI_freetuptable(tuptable);
 			memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
 			return TCL_RETURN;
 		}
 		if (loop_rc == TCL_BREAK)
 			break;
+		SPI_freetuptable(tuptable);
 		memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
 		return TCL_ERROR;
 	}
 
+	SPI_freetuptable(tuptable);
+
 	/************************************************************
 	 * Finally return the number of tuples
 	 ************************************************************/
 	memcpy(&Warn_restart, &save_restart, sizeof(Warn_restart));
-	sprintf(buf, "%d", ntuples);
+	snprintf(buf, sizeof(buf), "%d", ntuples);
 	Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	return TCL_OK;
 }
@@ -2242,7 +2283,7 @@ pltcl_SPI_lastoid(ClientData cdata, Tcl_Interp *interp,
 {
 	char		buf[64];
 
-	sprintf(buf, "%u", SPI_lastoid);
+	snprintf(buf, sizeof(buf), "%u", SPI_lastoid);
 	Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	return TCL_OK;
 }
@@ -2284,7 +2325,7 @@ pltcl_set_tuple_values(Tcl_Interp *interp, char *arrayname,
 	{
 		arrptr = &arrayname;
 		nameptr = &attname;
-		sprintf(buf, "%d", tupno);
+		snprintf(buf, sizeof(buf), "%d", tupno);
 		Tcl_SetVar2(interp, arrayname, ".tupno", buf, 0);
 	}
 
